@@ -38,6 +38,7 @@ from ironic.common import exception
 from ironic.common import faults
 from ironic.common.i18n import _
 from ironic.common import policy
+from ironic.common import state_machine
 from ironic.common import states
 from ironic.common import utils
 from ironic.conductor import steps as conductor_steps
@@ -95,6 +96,10 @@ TRAITS_SCHEMA = {
     ]
 }
 
+# NOTE(TheJulia): Support for vtep-physical-switch and
+# vtep-logical-switch has been removed as of API version 1.105,
+# but is still permitted on the API surface in the version range
+# so we don't break overall API interaction compatibility.
 LOCAL_LINK_BASE_SCHEMA = {
     'type': 'object',
     'properties': {
@@ -452,6 +457,8 @@ def sanitize_dict(to_sanitize, fields):
             to_sanitize.pop(key, None)
 
 
+# TODO(stephenfin): We can repurpose this check to only set a default for limit
+# when all APIs use jsonschema validation
 def validate_limit(limit):
     if limit is None:
         return CONF.api.max_limit
@@ -462,6 +469,8 @@ def validate_limit(limit):
     return min(CONF.api.max_limit, limit)
 
 
+# TODO(stephenfin): We can remove this check when all APIs use jsonschema
+# validation
 def validate_sort_dir(sort_dir):
     if sort_dir not in ['asc', 'desc']:
         raise exception.ClientSideError(_("Invalid sort direction: %s. "
@@ -893,6 +902,8 @@ VERSIONED_FIELDS = {
     'firmware_interface': versions.MINOR_86_FIRMWARE_INTERFACE,
     'service_step': versions.MINOR_87_SERVICE,
     'disable_power_off': versions.MINOR_95_DISABLE_POWER_OFF,
+    'instance_name': versions.MINOR_104_NODE_INSTANCE_NAME,
+    'health': versions.MINOR_109_NODE_HEALTH,
 }
 
 for field in V31_FIELDS:
@@ -952,7 +963,7 @@ def check_for_invalid_state_and_allow_filter(provision_state):
         if (api.request.version.minor
                 < versions.MINOR_9_PROVISION_STATE_FILTER):
             raise exception.NotAcceptable()
-        valid_states = states.machine.states
+        valid_states = state_machine.machine.states
         if provision_state not in valid_states:
             raise exception.InvalidParameterValue(
                 _('Provision state "%s" is not valid') % provision_state)
@@ -1165,6 +1176,20 @@ def check_allow_filter_by_shard(shard):
              'opr': versions.MINOR_82_NODE_SHARD})
 
 
+def check_allow_portgroup_filter_by_shard(shard):
+    """Check if filtering portgroups by shard is allowed.
+
+    Version 1.106 of the API allows filtering portgroups by shard.
+    """
+    if (shard is not None and api.request.version.minor
+            < versions.MINOR_106_PORTGROUP_SHARD):
+        raise exception.NotAcceptable(_(
+            "Request not acceptable. The minimal required API version "
+            "should be %(base)s.%(opr)s") %
+            {'base': versions.BASE_VERSION,
+             'opr': versions.MINOR_106_PORTGROUP_SHARD})
+
+
 def check_allow_child_node_params(include_children=None,
                                   parent_node=None):
     if ((include_children is not None
@@ -1289,6 +1314,26 @@ def allow_portgroup_mode_properties():
             >= versions.MINOR_26_PORTGROUP_MODE_PROPERTIES)
 
 
+def allow_portgroup_physical_network():
+    """Check if physical_network can be added to/queried from a portgroup.
+
+    Version 1.102 of the API added physical_network field to portgroup
+    object.
+    """
+    return (api.request.version.minor
+            >= versions.MINOR_102_PORTGROUP_PHYSICAL_NETWORK)
+
+
+def allow_portgroup_category():
+    """Check if category can be added to/queried from a portgroup.
+
+    Version 1.103 of the API added category field to portgroup
+    object.
+    """
+    return (api.request.version.minor
+            >= versions.MINOR_103_PORTGROUP_CATEGORY)
+
+
 def allow_vifs_subcontroller():
     """Check if node/vifs can be used.
 
@@ -2024,6 +2069,7 @@ def check_volume_policy_and_retrieve(policy_name, vol_ident, target=False):
     """
     context = api.request.context
     cdict = context.to_policy_values()
+    rpc_node = None
     owner = None
     lessee = None
     try:
@@ -2064,8 +2110,12 @@ def allow_ovn_vtep_version():
 
     Version 1.90 of the API added support for ovn
     vtep switches in port.local_link_connection.
+
+    Version 1.105 removed VTEP support logic.
     """
-    return api.request.version.minor >= versions.MINOR_90_OVN_VTEP
+    ver = api.request.version.minor
+    return (ver >= versions.MINOR_90_OVN_VTEP
+            and ver < versions.MINOR_105_REMOVE_OVN_VTEP)
 
 
 def allow_build_configdrive():
@@ -2085,14 +2135,6 @@ def allow_configdrive_vendor_data():
             >= versions.MINOR_59_CONFIGDRIVE_VENDOR_DATA)
 
 
-def allow_allocation_update():
-    """Check if updating an existing allocation is allowed or not.
-
-    Version 1.57 of the API added support for updating an allocation.
-    """
-    return api.request.version.minor >= versions.MINOR_57_ALLOCATION_UPDATE
-
-
 def allow_allocation_backfill():
     """Check if backfilling allocations is allowed.
 
@@ -2182,15 +2224,30 @@ def check_allow_boot_mode(node_capabilities, disallowed_boot_modes):
                                                    op=_('provisioning'))
 
 
-def check_allow_clean_disable_ramdisk(target, disable_ramdisk):
+def check_allow_disable_ramdisk(target, disable_ramdisk):
     if disable_ramdisk is None:
         return
-    elif api.request.version.minor < versions.MINOR_70_CLEAN_DISABLE_RAMDISK:
-        raise exception.NotAcceptable(
-            _("disable_ramdisk is not acceptable in this API version"))
-    elif target != "clean":
-        raise exception.BadRequest(
-            _("disable_ramdisk is supported only with manual cleaning"))
+
+    minor = api.request.version.minor
+    if target == states.VERBS['clean']:
+        if minor < versions.MINOR_70_CLEAN_DISABLE_RAMDISK:
+            raise exception.NotAcceptable(
+                _("disable_ramdisk is not acceptable in this API version"))
+        return
+
+    if target == states.VERBS['service']:
+        if minor < versions.MINOR_108_SERVICE_DISABLE_RAMDISK:
+            raise exception.NotAcceptable(
+                _("disable_ramdisk is not acceptable in this API version"))
+        return
+
+    raise exception.BadRequest(
+        _("disable_ramdisk is supported only with manual cleaning "
+          "or servicing"))
+
+
+def check_allow_clean_disable_ramdisk(target, disable_ramdisk):
+    return check_allow_disable_ramdisk(target, disable_ramdisk)
 
 
 def allow_shards_endpoint():
@@ -2250,3 +2307,33 @@ def allow_port_description():
     Version 1.97 of the API added description field to the port object.
     """
     return api.request.version.minor >= versions.MINOR_97_PORT_DESCRIPTION
+
+
+def allow_port_vendor():
+    """Check if vendor is allowed for ports.
+
+    Version 1.100 of the API added vendor field to the port object.
+    """
+    return api.request.version.minor >= versions.MINOR_100_PORT_VENDOR
+
+
+def allow_port_category():
+    """Check if category is allowed for ports.
+
+    Version 1.101 of the API added category field to the port object.
+    """
+    return api.request.version.minor >= versions.MINOR_101_PORT_CATEGORY
+
+
+def allow_node_instance_name():
+    """Check if instance_name is allowed for nodes.
+
+    Version 1.104 of the API added instance_name field to the node object.
+    """
+    return api.request.version.minor >= versions.MINOR_104_NODE_INSTANCE_NAME
+
+
+def check_allow_filter_by_instance_name(instance_name):
+    if instance_name is not None and not allow_node_instance_name():
+        raise exception.NotAcceptable(
+            _("instance_name is not acceptable in this API version"))
