@@ -20,13 +20,13 @@ Client side of the conductor RPC API.
 
 import random
 
-from ironic_lib.json_rpc import client as json_rpc
 from oslo_log import log
 import oslo_messaging as messaging
 
 from ironic.common import exception
 from ironic.common import hash_ring
 from ironic.common.i18n import _
+from ironic.common.json_rpc import client as json_rpc
 from ironic.common import release_mappings as versions
 from ironic.common import rpc
 from ironic.conf import CONF
@@ -157,12 +157,16 @@ class ConductorAPI(object):
     |    1.56 - Added continue_inspection
     |    1.57 - Added do_node_service
     |    1.58 - Added support for json-rpc port usage
+    |    1.59 - Added support for attaching/detaching virtual media
+    |    1.60 - Added continue_node_service
+    |    1.61 - Added get virtual media support
+    |    1.62 - Added update_portgroup_physical_network
     """
 
     # NOTE(rloo): This must be in sync with manager.ConductorManager's.
     # NOTE(pas-ha): This also must be in sync with
     #               ironic.common.release_mappings.RELEASE_MAPPING['master']
-    RPC_API_VERSION = '1.58'
+    RPC_API_VERSION = '1.62'
 
     def __init__(self, topic=None):
         super(ConductorAPI, self).__init__()
@@ -205,7 +209,7 @@ class ConductorAPI(object):
         # the hostname to match it against the current host.
         host = topic[len(self.topic) + 1:]
 
-        if rpc.GLOBAL_MANAGER is not None and host == CONF.host:
+        if self.client is None and host == CONF.host:
             # Short-cut to a local function call if there is a built-in
             # conductor.
             return _LOCAL_CONTEXT
@@ -235,7 +239,8 @@ class ConductorAPI(object):
             return dest.pop()
         except exception.DriverNotFound:
             reason = (_('No conductor service registered which supports '
-                        'driver %(driver)s for conductor group "%(group)s".') %
+                        'driver %(driver)s for conductor group "%(group)s". '
+                        'Ensure the driver is valid and enabled.') %
                       {'driver': node.driver, 'group': node.conductor_group})
             raise exception.NoValidHost(reason=reason)
 
@@ -622,6 +627,20 @@ class ConductorAPI(object):
         return cctxt.cast(context, 'continue_node_deploy',
                           node_id=node_id)
 
+    def continue_node_service(self, context, node_id, topic=None):
+        """Signal to conductor service to start the next service action.
+
+        NOTE(janders): this is an RPC cast, there will be no response or
+        exception raised by the conductor for this RPC.
+
+        :param context: request context.
+        :param node_id: node id or uuid.
+        :param topic: RPC topic. Defaults to self.topic.
+        """
+        cctxt = self._prepare_call(topic=topic, version='1.60')
+        return cctxt.cast(context, 'continue_node_service',
+                          node_id=node_id)
+
     def validate_driver_interfaces(self, context, node_id, topic=None):
         """Validate the `core` and `standardized` interfaces for drivers.
 
@@ -1437,3 +1456,86 @@ class ConductorAPI(object):
             node_id=node_id,
             service_steps=service_steps,
             disable_ramdisk=disable_ramdisk)
+
+    def attach_virtual_media(self, context, node_id, device_type, image_url,
+                             image_download_source=None, topic=None):
+        """Attach a virtual media device to the node.
+
+        :param context: request context.
+        :param node_id: node ID or UUID.
+        :param image_url: URL of the image to attach, HTTP or HTTPS.
+        :param image_download_source: Which way to serve the image to the BMC:
+            "http" to serve it from the provided location, "local" to serve
+            it from the local web server.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: UnsupportedDriverExtension if the driver does not support
+                 this call.
+        :raises: InvalidParameterValue if validation of management driver
+                 interface failed.
+        :raises: NodeLocked if node is locked by another conductor.
+        :raises: NoFreeConductorWorker when there is no free worker to start
+                 async task.
+
+        """
+        cctxt = self._prepare_call(topic=topic, version='1.59')
+        return cctxt.call(
+            context, 'attach_virtual_media',
+            node_id=node_id,
+            device_type=device_type,
+            image_url=image_url)
+
+    def detach_virtual_media(self, context, node_id, device_types=None,
+                             topic=None):
+        """Detach some or all virtual media devices from the node.
+
+        :param context: request context.
+        :param node_id: node ID or UUID.
+        :param device_types: A collection of device type, ones from
+            :data:`ironic.common.boot_devices.VMEDIA_DEVICES`.
+            If not provided, all devices are detached.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: UnsupportedDriverExtension if the driver does not support
+                 this call.
+        :raises: InvalidParameterValue if validation of management driver
+                 interface failed.
+        :raises: NodeLocked if node is locked by another conductor.
+        :raises: NoFreeConductorWorker when there is no free worker to start
+                 async task.
+
+        """
+        cctxt = self._prepare_call(topic=topic, version='1.59')
+        return cctxt.call(
+            context, 'detach_virtual_media',
+            node_id=node_id,
+            device_types=device_types)
+
+    def get_virtual_media(self, context, node_id, topic=None):
+        """Get all virtual media devices from the node.
+
+        :param context: request context.
+        :param node_id: node ID or UUID.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: UnsupportedDriverExtension if the driver does not support
+                 this call.
+        :raises: InvalidParameterValue if validation of management driver
+                 interface failed.
+        :raises: NodeLocked if node is locked by another conductor.
+
+        """
+        cctxt = self._prepare_call(topic=topic, version='1.61')
+        return cctxt.call(
+            context, 'get_virtual_media',
+            node_id=node_id)
+
+    def update_portgroup_physical_network(self, context, portgroup_obj,
+                                          new_physical_network, topic=None):
+        """Update the physical_network for a given portgroup and its ports.
+
+        :param context: request context.
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: NodeLocked if node is locked by another conductor.
+        """
+        cctxt = self._prepare_call(topic=topic, version='1.62')
+        return cctxt.call(context, 'update_portgroup_physical_network',
+                          portgroup=portgroup_obj,
+                          new_physical_network=new_physical_network)
