@@ -15,7 +15,7 @@
 
 import collections
 
-from neutronclient.common import exceptions as neutron_exceptions
+from openstack.connection import exceptions as openstack_exc
 from oslo_config import cfg
 from oslo_log import log
 
@@ -46,8 +46,7 @@ def _vif_attached(port_like_obj, vif_id):
         False otherwise.
     :raises: VifAlreadyAttached, if vif_id is attached to port_like_obj.
     """
-    attached_vif_id = port_like_obj.internal_info.get(
-        TENANT_VIF_KEY, port_like_obj.extra.get('vif_port_id'))
+    attached_vif_id = port_like_obj.internal_info.get(TENANT_VIF_KEY)
     if attached_vif_id == vif_id:
         raise exception.VifAlreadyAttached(
             object_type=port_like_obj.__class__.__name__,
@@ -233,9 +232,7 @@ def plug_port_to_tenant_network(task, port_like_obj, client=None):
     local_group_info = None
     client_id_opt = None
 
-    vif_id = (
-        port_like_obj.internal_info.get(TENANT_VIF_KEY)
-        or port_like_obj.extra.get('vif_port_id'))
+    vif_id = port_like_obj.internal_info.get(TENANT_VIF_KEY)
 
     if not vif_id:
         obj_name = port_like_obj.__class__.__name__.lower()
@@ -267,47 +264,125 @@ def plug_port_to_tenant_network(task, port_like_obj, client=None):
     # NOTE(sambetts) Only update required binding: attributes,
     # because other port attributes may have been set by the user or
     # nova.
-    body = {
-        'port': {
-            'binding:vnic_type': neutron.VNIC_BAREMETAL,
-            'binding:host_id': node.uuid,
-            'mac_address': port_like_obj.address
-        }
-    }
+    port_attrs = {'binding:vnic_type': neutron.VNIC_BAREMETAL,
+                  'binding:host_id': node.uuid}
+    # NOTE(kaifeng) Only update mac address when it's available
+    if port_like_obj.address:
+        port_attrs['mac_address'] = port_like_obj.address
     binding_profile = {'local_link_information': local_link_info}
     if local_group_info:
         binding_profile['local_group_information'] = local_group_info
-    body['port']['binding:profile'] = binding_profile
+
+    # Include physical_network if available
+    if port_like_obj.physical_network:
+        binding_profile['physical_network'] = port_like_obj.physical_network
+
+    port_attrs['binding:profile'] = binding_profile
 
     if client_id_opt:
-        body['port']['extra_dhcp_opts'] = [client_id_opt]
+        port_attrs['extra_dhcp_opts'] = [client_id_opt]
 
     is_smart_nic = neutron.is_smartnic_port(port_like_obj)
     if is_smart_nic:
         link_info = local_link_info[0]
         LOG.debug('Setting hostname as host_id in case of Smart NIC, '
                   'port %(port_id)s, hostname %(hostname)s',
-                  {'port_id': vif_id,
-                   'hostname': link_info['hostname']})
-        body['port']['binding:host_id'] = link_info['hostname']
-        body['port']['binding:vnic_type'] = neutron.VNIC_SMARTNIC
+                  {'port_id': vif_id, 'hostname': link_info['hostname']})
+        port_attrs['binding:host_id'] = link_info['hostname']
+        port_attrs['binding:vnic_type'] = neutron.VNIC_SMARTNIC
 
     if not client:
         client = neutron.get_client(context=task.context)
 
     if is_smart_nic:
-        neutron.wait_for_host_agent(client, body['port']['binding:host_id'])
+        neutron.wait_for_host_agent(client, port_attrs['binding:host_id'])
 
     try:
-        neutron.update_neutron_port(task.context, vif_id, body)
-        if is_smart_nic:
-            neutron.wait_for_port_status(client, vif_id, 'ACTIVE')
-    except neutron_exceptions.ConnectionFailed as e:
+        neutron.update_neutron_port(task.context, vif_id, port_attrs)
+
+        binding_fail_fatal = False
+        is_neutron_iface = node.network_interface == 'neutron'
+
+        if is_neutron_iface:
+            binding_fail_fatal = getattr(
+                CONF.neutron, 'fail_on_port_binding_failure', False)
+
+        default_failure_behavior = is_smart_nic or binding_fail_fatal
+
+        fail_on_binding_failure = node.driver_info.get(
+            'fail_on_binding_failure', default_failure_behavior)
+
+        # NOTE(cid): Only check port status if it's a smart NIC or if we're
+        # configured to fail on binding failures. This avoids unnecessary
+        # failures when using network interfaces where binding may fail but
+        # we want to continue anyway (like in the case of flat networks).
+        if fail_on_binding_failure:
+            neutron.wait_for_port_status(
+                client, vif_id, 'ACTIVE',
+                fail_on_binding_failure=fail_on_binding_failure)
+    except openstack_exc.OpenStackCloudException as e:
         msg = (_('Could not add public network VIF %(vif)s '
                  'to node %(node)s, possible network issue. %(exc)s') %
-               {'vif': vif_id,
-                'node': node.uuid,
-                'exc': e})
+               {'vif': vif_id, 'node': node.uuid, 'exc': e})
+        LOG.error(msg)
+        raise exception.NetworkError(msg)
+
+
+def update_port_host_id(task, vif_id, client=None):
+    """Send an initial host_id value to Neutron to enable allocation.
+
+    :param task: A TaskManager instance.
+    :param vif_id: The VIF ID to "attach" to the host.
+    :param client: A neutron client object
+    :raises: NetworkError if failed to update Neutron port.
+    :raises: VifNotAttached if tenant VIF is not associated with port_like_obj.
+    """
+
+    node = task.node
+
+    if not vif_id:
+        # Nothing to do here, there *is* no VIF to update.
+        return
+
+    if not client:
+        client = neutron.get_client(task.context)
+
+    LOG.debug('Seeding network configuration for VIF %(vif_id)s to node '
+              '%(node_id)s',
+              {'vif_id': vif_id, 'node_id': node.uuid})
+    # We cannot perform an initial/early bind if the port is already
+    # bound. For example, Metalsmith users pre-pass the host_id to
+    # neutron, but then causes this patch to fail in such a case.
+    neutron.unbind_neutron_port_if_bound(vif_id, context=task.context,
+                                         client=client)
+
+    # NOTE(TheJulia): Just provide enough data through to neutron
+    # to start initial binding, but ultimately this entire binding
+    # sequence won't succeed because it cannot succeed. No binding
+    # profile is included, so an ML2 plugin cannot act on the change
+    # and effectively neutron should treat the port update as a NOOP,
+    # except if deferred addressing is the case, then that should
+    # trigger.
+    port_attrs = {'binding:vnic_type': neutron.VNIC_BAREMETAL,
+                  'binding:host_id': node.uuid,
+                  'binding:profile': {}}
+    try:
+        # This cannot be done with the prior/existing client, because they
+        # bring a different context and thus API rights. In other words,
+        # we, as an adminy service need to do this.
+        neutron.update_neutron_port(task.context, vif_id, port_attrs,
+                                    client=None)
+    except openstack_exc.OpenStackCloudException as e:
+        # If his action has failed, the possibility exists that the port
+        # was previously bound *and* allocated to another network, and thus
+        # cannot be properly bound later *either*. This exception should
+        # cause the deployment flow to never be triggered if there is an
+        # underlying issue which only neutron can see like wrong segment
+        # for the vif.
+        msg = (_('Could not seed network configuration for VIF %(vif)s'
+                 'to node %(node)s, possible network, state, or access '
+                 'permission issue. %(exc)s') %
+               {'vif': vif_id, 'node': node.uuid, 'exc': e})
         LOG.error(msg)
         raise exception.NetworkError(msg)
 
@@ -370,10 +445,7 @@ class VIFPortIDMixin(object):
         :param port_like_obj: A port or portgroup to check.
         :returns: The ID of the attached VIF, or None.
         """
-        # FIXME(sambetts) Remove this when we no longer support a nova
-        # driver that uses port.extra
-        return (port_like_obj.internal_info.get(TENANT_VIF_KEY)
-                or port_like_obj.extra.get('vif_port_id'))
+        return port_like_obj.internal_info.get(TENANT_VIF_KEY)
 
     def vif_list(self, task):
         """List attached VIF IDs for a node
@@ -409,30 +481,6 @@ class VIFPortIDMixin(object):
                 or p_obj.internal_info.get('inspection_vif_port_id')
                 or self._get_vif_id_by_port_like_obj(p_obj) or None)
 
-    def get_node_network_data(self, task):
-        """Return network configuration for node NICs.
-
-        Gather L2 and L3 network settings from ironic node `network_data`
-        field. Ironic would eventually pass network configuration to the node
-        being managed out-of-band.
-
-        :param task: A TaskManager instance.
-        :raises: InvalidParameterValue, if the network interface configuration
-            is invalid.
-        :raises: MissingParameterValue, if some parameters are missing.
-        :returns: a dict holding network configuration information adhearing
-            Nova network metadata layout (`network_data.json`).
-        """
-        node = task.node
-
-        network_data = node.network_data
-
-        # TODO(etingof): remove or truncate `network_data` logging
-        LOG.debug('Collected network data for node %(node)s: %(data)s',
-                  {'node': node.uuid, 'data': network_data})
-
-        return network_data
-
 
 class NeutronVIFPortIDMixin(VIFPortIDMixin):
     """VIF port ID mixin class for neutron network interfaces.
@@ -467,8 +515,7 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
             original_port = objects.Port.get_by_id(context, port_obj.id)
             updated_client_id = port_obj.extra.get('client-id')
 
-            if (original_port.extra.get('client-id')
-                != updated_client_id):
+            if original_port.extra.get('client-id') != updated_client_id:
                 # DHCP Option with opt_value=None will remove it
                 # from the neutron port
                 if vif:
@@ -485,8 +532,7 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
                         "No VIF found for instance %(instance)s "
                         "port %(port)s when attempting to update port "
                         "client-id.",
-                        {'port': port_uuid,
-                         'instance': node.instance_uuid})
+                        {'port': port_uuid, 'instance': node.instance_uuid})
 
         if portgroup_obj and ((set(port_obj.obj_what_changed())
                               & {'pxe_enabled', 'portgroup_id'}) or vif):
@@ -572,25 +618,7 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
         vif_id = vif_info['id']
         client = neutron.get_client(context=task.context)
 
-        # Determine whether any of the node's ports have a physical network. If
-        # not, we don't need to check the VIF's network's physical networks as
-        # they will not affect the VIF to port mapping.
-        physnets = set()
-        if any(port.physical_network is not None for port in task.ports):
-            physnets = neutron.get_physnets_by_port_uuid(client, vif_id)
-
-            if len(physnets) > 1:
-                # NOTE(mgoddard): Neutron cannot currently handle hosts which
-                # are mapped to multiple segments in the same routed network.
-                node_physnets = network.get_physnets_for_node(task)
-                if len(node_physnets.intersection(physnets)) > 1:
-                    reason = _("Node has ports which map to multiple segments "
-                               "of the routed network to which the VIF is "
-                               "attached. Currently neutron only supports "
-                               "hosts which map to one segment of a routed "
-                               "network")
-                    raise exception.VifInvalidForAttach(
-                        node=task.node.uuid, vif=vif_id, reason=reason)
+        physnets = neutron.get_physnets_by_port_uuid(client, vif_id)
 
         port_like_obj = get_free_port_like_object(
             task, vif_id, physnets, vif_info)
@@ -613,6 +641,10 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
         # NOTE(vsaienko) allow to attach VIF to active instance.
         if task.node.provision_state == states.ACTIVE:
             plug_port_to_tenant_network(task, port_like_obj, client=client)
+        else:
+            # Sends *just* a host_id to trigger neutron to do the initial
+            # address work (if needed...)
+            update_port_host_id(task, vif_id, client=client)
 
     def vif_detach(self, task, vif_id):
         """Detach a virtual network interface from a node
@@ -631,5 +663,60 @@ class NeutronVIFPortIDMixin(VIFPortIDMixin):
         # NOTE(vsaienko): allow to unplug VIFs from ACTIVE instance.
         # NOTE(TheJulia): Also ensure that we delete the vif when in
         # DELETING state.
-        if task.node.provision_state in [states.ACTIVE, states.DELETING]:
+        if task.node.provision_state in [states.ACTIVE, states.DELETING,
+                                         states.AVAILABLE]:
             neutron.unbind_neutron_port(vif_id, context=task.context)
+
+    def get_node_network_data(self, task):
+        """Get network configuration data for node ports.
+
+        Pull network data from ironic node object if present, otherwise
+        collect it for Neutron VIFs. The intended use of this method is to
+        generate data *for* virtual media based deployments so a minimal
+        configuration can be injected into the ramdisk.
+
+        :param task: A TaskManager instance.
+        :raises: InvalidParameterValue, if the network interface configuration
+            is invalid.
+        :raises: MissingParameterValue, if some parameters are missing.
+        :returns: a dict holding network configuration information adhering
+            Nova network metadata layout (`network_data.json`).
+        """
+        # NOTE(etingof): static network data takes precedence
+        network_data = (
+            super(NeutronVIFPortIDMixin, self).get_node_network_data(task))
+        if network_data:
+            return network_data
+
+        node = task.node
+
+        LOG.debug('Gathering network data from ports of node '
+                  '%(node)s', {'node': node.uuid})
+
+        network_data = collections.defaultdict(list)
+
+        for port_obj in task.ports:
+            # FIXME(TheJulia) This code ignores portgroups, but also nobody
+            # has complained about this issue *and* it uses get_current_vif
+            # which doesn't handle *tenant* vifs. The model of this is
+            # around deployment, not instances, really.
+            vif_port_id = self.get_current_vif(task, port_obj)
+
+            LOG.debug('Considering node %(node)s port %(port)s, VIF %(vif)s',
+                      {'node': node.uuid, 'port': port_obj.uuid,
+                       'vif': vif_port_id})
+
+            if not vif_port_id:
+                continue
+
+            port_network_data = neutron.get_neutron_port_data(
+                port_obj.uuid, vif_port_id, context=task.context)
+
+            for field, field_data in port_network_data.items():
+                if field_data:
+                    network_data[field].extend(field_data)
+
+        LOG.debug('Collected network data for node %(node)s: %(data)s',
+                  {'node': node.uuid, 'data': network_data})
+
+        return network_data
